@@ -28,6 +28,7 @@ def test_go_rule02():
     a[3] = "NOT"
     a[4] = "GO:0005515"
     a[8] = "F"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule02().test(assoc, assocparser.AssocParserConfig())
@@ -48,6 +49,8 @@ def test_go_rule_06():
     a[4] = "GO:0005575" # Cellular component
     a[6] = "HEP"
     a[8] = "C"
+    a[13] = "20200303"
+
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule06().test(assoc, assocparser.AssocParserConfig(ontology=ontology))
@@ -68,6 +71,7 @@ def test_go_rule_07():
     a[4] = "GO:0003824"
     a[6] = "IPI"
     a[8] = "F"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule07().test(assoc, assocparser.AssocParserConfig(ontology=ontology))
@@ -88,6 +92,7 @@ def test_go_rule08():
     a[4] = "GO:0006810" # do not annotate
     a[6] = "IEA"
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule08().test(assoc, assocparser.AssocParserConfig(ontology=ontology))
@@ -118,6 +123,7 @@ def test_go_rule11():
     a[4] = "GO:0003674"
     a[6] = "ND"
     a[8] = "F"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule11().test(assoc, assocparser.AssocParserConfig())
@@ -129,6 +135,7 @@ def test_go_rule11():
     a[4] = "GO:1234567"
     a[6] = "ND"
     a[8] = "F"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule11().test(assoc, assocparser.AssocParserConfig())
@@ -139,6 +146,7 @@ def test_go_rule11():
     a[4] = "GO:1234567"
     a[6] = "IEA"
     a[8] = "F"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule11().test(assoc, assocparser.AssocParserConfig())
@@ -150,6 +158,7 @@ def test_go_rule11():
     a[4] = "GO:0003674"
     a[6] = "IEA"
     a[8] = "F"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule11().test(assoc, assocparser.AssocParserConfig())
@@ -175,6 +184,12 @@ def test_go_rules_13():
     test_result = qc.GoRule13().test(assoc, assocparser.AssocParserConfig(annotation_inferences=gaferences))
     assert test_result.result_type == qc.ResultType.PASS
 
+    a = ["AspGD", "ASPL0000059928", "AN0127", "", "GO:0032258", "AspGD_REF:ASPL0000000005", "IEA", "SGD:S000001917", "P", "", "AN0127|ANID_00127|ANIA_00127", "gene_product", "taxon:227321", "20200201", "AspGD", "", ""]
+    assoc = gafparser.to_association(a).associations[0]
+    gaferences = gaference.load_gaferencer_inferences_from_file("tests/resources/test.inferences.json")
+    test_result = qc.GoRule13().test(assoc, assocparser.AssocParserConfig(annotation_inferences=gaferences))
+    assert test_result.result_type == qc.ResultType.ERROR
+
 def test_go_rules_15():
 
     a = ["blah"] * 15
@@ -183,6 +198,7 @@ def test_go_rules_15():
     a[6] = "IEA"
     a[8] = "P"
     a[12] = "taxon:123|taxon:456"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule15().test(assoc, assocparser.AssocParserConfig(ontology=ontology))
@@ -206,6 +222,7 @@ def test_go_rule_16():
     a[6] = "IC"
     a[7] = "BLAH:12345"
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule16().test(assoc, assocparser.AssocParserConfig())
@@ -244,6 +261,7 @@ def test_go_rule_17():
     a[6] = "IDA"
     a[7] = "BLAH:12345"
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule17().test(assoc, assocparser.AssocParserConfig())
@@ -261,6 +279,7 @@ def test_go_rule_18():
     a[6] = "IPI"
     a[7] = ""
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
     assoc.evidence.with_support_from = []
 
@@ -281,6 +300,7 @@ def test_go_rule26():
     a = make_annotation("GO:BLAHBLAH", "IBA")
     a[8] = "P"
     a[3] = ""
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
     # Pass due to IBA in paint
     test_result = qc.GoRule26().test(assoc, config)
@@ -293,6 +313,7 @@ def test_go_rule26():
     a = make_annotation("GO:BLAHBLAH", "IPI")
     a[8] = "P"
     a[3] = ""
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
     # Pass due to non IBA
     test_result = qc.GoRule26().test(assoc, config)
@@ -305,6 +326,7 @@ def test_go_rule26():
     a = make_annotation("GO:BLAHBLAH", "IBA")
     a[8] = "P"
     a[3] = ""
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
     # Pass due to non IBA
     test_result = qc.GoRule26().test(assoc, config)
@@ -320,6 +342,7 @@ def test_go_rule28():
     a[3] = ""
     a[4] = "GO:0005975"
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule28().test(assoc, config)
@@ -331,6 +354,7 @@ def test_go_rule28():
     a[3] = ""
     a[4] = "GO:0005975"
     a[8] = "C"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule28().test(assoc, config)
@@ -349,27 +373,35 @@ def test_go_rule29():
     a[13] = "19901111" # Nov 11, 1990, more than a year old
     assoc = gafparser.to_association(a).associations[0]
 
+    ## Base test: old IEA.
     test_result = qc.GoRule29().test(assoc, assocparser.AssocParserConfig())
     assert test_result.result_type == qc.ResultType.ERROR
 
+    ## Pass if not IEA
     assoc.evidence.type = "ECO:0000305" #Not IEA
-
     test_result = qc.GoRule29().test(assoc, assocparser.AssocParserConfig())
     assert test_result.result_type == qc.ResultType.PASS
 
+    ## Pass if only a half year old.
     now = datetime.datetime.now()
     six_months_ago = now - datetime.timedelta(days=180)
     assoc.date = six_months_ago.strftime("%Y%m%d")
     assoc.evidence.type = "ECO:0000501"
-
     test_result = qc.GoRule29().test(assoc, assocparser.AssocParserConfig())
     assert test_result.result_type == qc.ResultType.PASS
 
+    ## Warning if a year and a half year old.
+    eighteen_months_ago = now - datetime.timedelta(days=(30*18))
+    assoc.date = eighteen_months_ago.strftime("%Y%m%d")
+    test_result = qc.GoRule29().test(assoc, assocparser.AssocParserConfig())
+    assert test_result.result_type == qc.ResultType.WARNING
+
 def test_gorule30():
     a = ["blah"] * 15
     a[3] = ""
     a[5] = "GO_REF:0000033"
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     config = assocparser.AssocParserConfig(
@@ -383,15 +415,15 @@ def test_gorule30():
     )
 
     test_result = qc.GoRule30().test(assoc, config)
-    assert test_result.result_type == qc.ResultType.ERROR
+    assert test_result.result_type == qc.ResultType.WARNING
 
     assoc.evidence.has_supporting_reference = ["GO_PAINT:0000000"]
     test_result = qc.GoRule30().test(assoc, config)
-    assert test_result.result_type == qc.ResultType.ERROR
+    assert test_result.result_type == qc.ResultType.WARNING
 
     assoc.evidence.has_supporting_reference = ["FOO:123", "GO_REF:0000033"]
     test_result = qc.GoRule30().test(assoc, config)
-    assert test_result.result_type == qc.ResultType.ERROR
+    assert test_result.result_type == qc.ResultType.WARNING
 
     assoc.evidence.has_supporting_reference = ["FOO:123"]
     test_result = qc.GoRule30().test(assoc, config)
@@ -403,6 +435,7 @@ def test_gorule37():
     a[6] = "IBA"
     a[5] = "PMID:21873635"
     a[8] = "P"
+    a[13] = "20200303"
     a[14] = "GO_Central"
     assoc = gafparser.to_association(a).associations[0]
 
@@ -429,6 +462,7 @@ def test_gorule39():
     a[3] = ""
     a[4] = "GO:0032991"
     a[8] = "C"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule39().test(assoc, assocparser.AssocParserConfig())
@@ -448,6 +482,7 @@ def test_gorule42():
     a[3] = "NOT"
     a[6] = "IKR"
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule42().test(assoc, assocparser.AssocParserConfig())
@@ -468,6 +503,7 @@ def test_gorule43():
     a[5] = "GO_REF:0000024"
     a[6] = "ISO"
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     config = assocparser.AssocParserConfig(
@@ -502,6 +538,7 @@ def test_gorule46():
     a[4] = "GO:0051260" # Self-binding, yes
     a[7] = "PomBase:SPAC25B8.17"
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule46().test(assoc, config)
@@ -533,6 +570,7 @@ def test_gorule50():
     a[6] = "ISS"
     a[7] = "HELLO:123"
     a[8] = "P"
+    a[13] = "20200303"
     assoc = gafparser.to_association(a).associations[0]
 
     test_result = qc.GoRule50().test(assoc, assocparser.AssocParserConfig())
@@ -549,21 +587,24 @@ def test_gorule50():
     assert test_result.result_type == qc.ResultType.PASS
 
 def test_gorule57():
-    a = ["blah"] * 12
+    a = ["blah"] * 13
     a[0] = "HELLO"
     a[1] = "123"
-    a[2] = "enables"
+    a[2] = "contributes_to"
     a[3] = "GO:0003674"
     a[4] = "PMID:12345"
     a[5] = "ECO:0000501"
     a[7] = "taxon:2"
-    # a[8] = "P"
+    a[8] = "20200303"
     a[9] = "MGI"
     a[10] = ""
     a[11] = ""
 
+    res = gpadparser.to_association(a)
+    print(json.dumps(res.report.to_report_json(), indent=4))
     assoc = gpadparser.to_association(a).associations[0]
 
+
     # Look at evidence_code, reference, annotation_properties
     config = assocparser.AssocParserConfig(group_metadata={
         "id": "mgi",
@@ -610,6 +651,7 @@ def test_gorule58():
     a[9] = "MGI"
     a[10] = ""
     a[11] = ""
+    a[13] = "20200303"
     a[14] = "GO_Central"
     a[15] = "has_input(GO:0003674),occurs_in(CL:123456)"
 
